import React, {useEffect, useState} from 'react'
import Login from "./components/Login";
import {useMount, useSetState} from "ahooks";
import {info, UserModel} from "./services/api";
import Vip from "./components/vip";
import Button from "./components/Button";
import {io} from 'socket.io-client'
import { useEffectOnce } from './utils/useEffectOnce';


type MessageItem = {
    id: number;
    from: 'ai' | 'you';
    time: number;
    content: string[];
    avatar: string;
}
const LOGIN_TOKEN_KEY = APP_CONFIG.LOGIN_TOKEN_KEY;
const avatar = 'https://www.cdnjson.com/images/2023/01/31/qwrgFf.jpg',
    avatar1 = 'https://www.cdnjson.com/images/2023/02/25/AI.png',
    globalMessageList: MessageItem[] = [
        {id: 1, from: 'you', time: 1, content: ['你好'], avatar},
        {id: 2, from: 'ai', time: 2, content: ['你好!有什么我能帮助你的吗?'], avatar: avatar1},
        {id: 3, from: 'you', time: 3, content: ['今天新闻'], avatar},
        {
            id: 4,
            from: 'ai',
            time: 4,
            content: ['很抱歉，我不会自己收集或浏览最新的新闻报道，因为我是一个人工智能语言模型，我的知识是基于我被训练的时间截止到2023年1月的。但是，您可以尝试搜索各大新闻网站来了解最新的新闻，例如CNN、BBC、纽约时报等等。'],
            avatar: avatar1
        },
    ],

    socket = io(APP_CONFIG.API_PREFIX, {
        autoConnect: false,
        path:`/socket.io`
    })
let messageIndex = 5, lastMessage = 0, initSocket = false;


function App() {
    const [count, setCount] = useState(3)
    const [state, setState] = useSetState<{
        showLogin?: boolean
        user?: UserModel
        showVip?: boolean
        message?: string
        sending?: boolean
    }>({
        showVip: false,
    })

    const [messageList, setMessageList] = useState<MessageItem[]>(globalMessageList)

    function logout() {
        //TODO 调用注销登录接口
        localStorage.removeItem(LOGIN_TOKEN_KEY)
        setState({user: undefined})
    }

    function initConnection() {
        if (socket.connected || initSocket) return;
        initSocket = true
        console.log('初始化链接')
        socket.on('disconnect', () => {
            initSocket = false
        })
        // 时间
        socket.on('chat', (ret: { content: string, messageId: number }) => {
            setState({message: ''})
            if (lastMessage != ret.messageId) {
                appendMessage('ai', ret.content)
            } else {
                appendToLastMessage(ret.content)
            }
            lastMessage = ret.messageId
        })
        socket.on('chat-end', () => {
            setState({sending: false})
            if (count > 1) {
                setCount(count - 1)
            }
        })
        socket.connect()
    }

    function appendToLastMessage(content: string) {
        const last = globalMessageList[globalMessageList.length - 1]
        last.content.push(content)
        setMessageList([...globalMessageList])
    }

    function loadUserInfo() {
        if (!localStorage.getItem(LOGIN_TOKEN_KEY)) {
            return;
        }
        info().then(user => {
            setState({user})
            initConnection()
        })
    }

    function appendMessage(from: 'you' | 'ai', content: string) {
        const newMsg = {
            id: messageIndex++,
            from,
            time: Date.now(),
            content: [content],
            avatar: from == 'you' ? avatar : avatar1
        }
        globalMessageList.push(newMsg)
        setMessageList([...globalMessageList])
    }

    function onSendChat() {
        if (!state.message) {
            return;
        }
        if (!state.user) { // 没有登录
            setState({showLogin: true})
            return;
        }
        if (count < 1 && state.user.vip < (Date.now() / 100) >> 0) {
            setState({showVip: true})
            return;
        }
        // 是否已经链接服务
        if (socket.disconnected) {
            console.log('服务未链接')
            return;
        }
        setState({sending: true})
        appendMessage('you', state.message)
        socket.emit('chat', {message: state.message})
        // sendChat(state.message).then((ret) => {
        //     setState({message: ''})
        //     appendMessage('ai', ret.content)
        // }).finally(() => setState({sending: false}))
    }

    const scrollToBottom = () => {
        (function smoothScroll() {
            const currentScroll = document.documentElement.scrollTop || document.body.scrollTop; // 已经被卷掉的高度
            const clientHeight = document.documentElement.clientHeight; // 浏览器高度
            const scrollHeight = document.documentElement.scrollHeight; // 总高度
            if (scrollHeight - 10 > currentScroll + clientHeight) {
                window.requestAnimationFrame(smoothScroll);
                window.scrollTo(0, currentScroll + (scrollHeight - currentScroll - clientHeight) / 2);
            }
        })();
    };
    // 监听
    useEffectOnce(loadUserInfo)
    useEffect(() => {
        if (messageList.length == 0) return;
        scrollToBottom();
    }, [messageList])

    return (
        <div className="app-wrapper">
            <div className="app-header">
                <div className="header flex-center">
                    {state.user ? (<>
                        <img className="avatar" src={state.user.avatar} alt=""/>
                        <span className="nickname">{state.user.username}</span>
                        <span className="line"></span>
                        <span onClick={logout} className="logout cursor-pointer">退出</span>
                    </>) : <span className="cursor-pointer" onClick={() => setState({showLogin: true})}>登录</span>}
                </div>
            </div>
            <div className="app-content">
                <div className="message-list-wrapper">
                    {messageList.map((msg, index) => <div className="message-item" key={index}>
                        <div className="content-container message-content">
                            <div className="avatar-container">
                                <img className="avatar" src={msg.avatar} alt=""/>
                            </div>
                            <div className="content">{msg.content.map((block, blockIndex) => <p
                                key={blockIndex}>{block}</p>)}</div>
                        </div>
                    </div>)}
                </div>
            </div>
            <div className="app-send-wrapper">
                <div className="content-container">
                    <form onSubmit={(e) => {
                        e.preventDefault();
                        onSendChat()
                    }}>
                        <textarea
                            onKeyUp={e => {
                                if (e.key.toLowerCase() == 'enter') {
                                    onSendChat()
                                    e.preventDefault();
                                    e.stopPropagation();
                                }
                            }}
                            onInput={e => setState({message: e.currentTarget.value})}
                            className="send-content"
                            placeholder="请输入..." title="" value={state.message}></textarea>
                        <div className="sender-wrapper">
                            <Button className="btn-sender" htmlType="submit" type="primary" disabled={state.sending}>
                                {state.sending ? '发送中...' : (<>
                                    <div>发 送</div>
                                    <div>剩余 {count} 次</div>
                                </>)}
                            </Button>
                            {state.user?.vip &&
                                <div className="open-vip" onClick={() => setState({showVip: true})}>+ 增加次数 &gt;</div>}
                        </div>
                    </form>
                </div>
            </div>
            {state.showLogin && <Login
                onClose={(logged,data) => {
                    setState({showLogin: false})
                    if(logged && data){
                        localStorage.setItem(LOGIN_TOKEN_KEY, data.token)
                        loadUserInfo()
                    }
                }}
            />}
            {state.showVip && <Vip onClose={(opened) => {
                setState({showVip: false})
                opened && loadUserInfo()
            }}/>}
        </div>
    )
}

export default App
